==> 1.6.1.tar.gz: lib/UnitRCDartRuntime.dart <==
import 'dart:async';
import 'dart:convert';
import 'dart:ffi'; // For FFI
import 'dart:io'; // For Platform.isX
import 'dart:isolate';
import 'dart:typed_data';
import 'package:ffi/ffi.dart';

import 'DartBase.dart';
import 'UnitRCDartNotify.dart';

//1. 加载 C++ 库
late final unitRCLib = Platform.isAndroid
    ? DynamicLibrary.open("libunit_rc.so")
    : DynamicLibrary.process();


int createReceivePort() {
  final interactiveCppRequests = ReceivePort()..listen(requestExecuteCallback);
  final int nativePort = interactiveCppRequests.sendPort.nativePort;
  return nativePort;
}

void registerPortToCPP(int sendPort) {
  final initializeApi = unitRCLib.lookupFunction<
      IntPtr Function(Pointer<Void>, Int64),
      int Function(Pointer<Void>, int)>("UnitRC_InitDartApiDL");
  final _dartAPIResult = initializeApi(NativeApi.initializeApiDLData, sendPort);
}

class UnitRCDartRuntime {
  UnitRCDartRuntime._();

  static Pointer<Void> getDartBindPointerFunc(Object object) {
    if (object is DartBaseClass) {
      return object.cppObjectPointer;
    }
    return nullptr;
  }
  static final getDartBindPointerFuncCFuntionPointer =
      Pointer.fromFunction<Pointer<Void> Function(Handle)>(
          getDartBindPointerFunc);
  static final registerGetDartBindPointerFunc =
      unitRCLib.lookupFunction<
              Void Function(
                  Pointer<NativeFunction<Pointer<Void> Function(Handle)>>),
              void Function(
                  Pointer<NativeFunction<Pointer<Void> Function(Handle)>>)>(
          "UnitRC_Register_GetDartBindPointerFunc");

  static void setDartBindPointer(Object object, Pointer<Void> pointer) {
    if (object is DartBaseClass) {
      object.cppObjectPointer = pointer;
    }
  }
  static final setDartBindPointerCFuntionPointer =
      Pointer.fromFunction<Void Function(Handle, Pointer<Void>)>(
          setDartBindPointer);
  static final registerSetDartBindPointerFunc = unitRCLib.lookupFunction<
          Void Function(
              Pointer<NativeFunction<Void Function(Handle, Pointer<Void>)>>),
          void Function(
              Pointer<NativeFunction<Void Function(Handle, Pointer<Void>)>>)>(
      "UnitRC_Register_SetDartBindPointerFunc");

  static Object? getDartNull() {
    return null;
  }
  static final getDartNullCFunctionPointer =
      Pointer.fromFunction<Handle Function()>(getDartNull);
  static final registerGetDartNullCFunctionPointer = unitRCLib.lookupFunction<
      Void Function(Pointer<NativeFunction<Handle Function()>>),
      void Function(
          Pointer<NativeFunction<Handle Function()>>)>("UnitRC_Register_GetDartNull");

  static bool isDartNull(Object? obj) {
    return obj == null;
  }
  static final isDartNullCFunctionPointer = Pointer.fromFunction<Bool Function(Handle)>(isDartNull, false);
  static final registerIsDartNullCFunctionPointer = unitRCLib.lookupFunction<
    Void Function(Pointer<NativeFunction<Bool Function(Handle)>>),
    void Function(Pointer<NativeFunction<Bool Function(Handle)>>)
  >("UnitRC_Register_IsDartNull");

  static Object getDartNullptr() {
    return nullptr;
  }
  static final getDartNullptrCFunctionPointer = Pointer.fromFunction<Handle Function()>(getDartNullptr);
  static final registerGetDartNullptrCFunctionPointer = unitRCLib.lookupFunction<
    Void Function(Pointer<NativeFunction<Handle Function()>>),
    void Function(Pointer<NativeFunction<Handle Function()>>)
  >("UnitRC_Register_GetDartNullptr");

  static int getDartHash(Object? obj) {
    return obj?.hashCode ?? 0;
  }
  static final getDartHashCFunctionPointer = Pointer.fromFunction<Int64 Function(Handle)>(getDartHash, 0);
  static final registerGetDartHashCFunctionPointer = unitRCLib.lookupFunction<
    Void Function(Pointer<NativeFunction<Int64 Function(Handle)>>),
    void Function(Pointer<NativeFunction<Int64 Function(Handle)>>)
  >("UnitRC_Register_GetDartHash");

  static int currentIsolate() {
    return Isolate.current.hashCode;
  }
  static final currentIsolateFP = Pointer.fromFunction<Int64 Function()>(currentIsolate, 0);
  static final registerCurrentIsolateFP = unitRCLib.lookupFunction<
    Void Function(Pointer<NativeFunction<Int64 Function()>>),
    void Function(Pointer<NativeFunction<Int64 Function()>>)
  >("UnitRC_Register_CurrentIsolate");


  static void unregisterDartThread() {
    unregisterDartThreadFP();
  }

  static final unregisterDartThreadFP = unitRCLib.lookupFunction<
      Void Function(),
      void Function()>("UnitRC_Unregister_DartThread");

  static bool _isRegistered = false;
  static void registerNativeMethods() {
    if (_isRegistered) {
      return;
    }
    _isRegistered = true;
    registerGetDartBindPointerFunc(getDartBindPointerFuncCFuntionPointer);
    registerSetDartBindPointerFunc(setDartBindPointerCFuntionPointer);
    registerGetDartNullCFunctionPointer(getDartNullCFunctionPointer);
    registerIsDartNullCFunctionPointer(isDartNullCFunctionPointer);
    registerGetDartNullptrCFunctionPointer(getDartNullptrCFunctionPointer);
    registerGetDartHashCFunctionPointer(getDartHashCFunctionPointer);
    registerCurrentIsolateFP(currentIsolateFP);
    registerPortToCPP(createReceivePort());
  }
}

==> 1.6.8.tar.gz: lib/UnitRCDartRuntime.dart <==
import 'dart:ffi'; // For FFI
import 'dart:io'; // For Platform.isX
import 'dart:isolate';
import 'package:unit_rc/UnitRCLog.dart';

import 'DartBase.dart';
import 'UnitRCDartNotify.dart';

//1. 加载 C++ 库
late final unitRCLib = Platform.isAndroid
    ? DynamicLibrary.open("libunit_rc.so")
    : DynamicLibrary.process();


int createReceivePort() {
  final interactiveCppRequests = ReceivePort()..listen(requestExecuteCallback);
  final int nativePort = interactiveCppRequests.sendPort.nativePort;
  return nativePort;
}

void registerPortToCPP(int sendPort) {
  final initializeApi = unitRCLib.lookupFunction<
      IntPtr Function(Pointer<Void>, Int64),
      int Function(Pointer<Void>, int)>("UnitRC_InitDartApiDL");
  final _dartAPIResult = initializeApi(NativeApi.initializeApiDLData, sendPort);
  UnitRCLog.info("registerPortToCPP result: $_dartAPIResult");
}

class UnitRCDartRuntime {
  UnitRCDartRuntime._();

  static Pointer<Void> getDartBindPointerFunc(Object object) {
    if (object is DartBaseClass) {
      return object.cppObjectPointer;
    }
    return nullptr;
  }
  static final getDartBindPointerFuncCFuntionPointer =
      Pointer.fromFunction<Pointer<Void> Function(Handle)>(
          getDartBindPointerFunc);
  static final registerGetDartBindPointerFunc =
      unitRCLib.lookupFunction<
              Void Function(
                  Pointer<NativeFunction<Pointer<Void> Function(Handle)>>),
              void Function(
                  Pointer<NativeFunction<Pointer<Void> Function(Handle)>>)>(
          "UnitRC_Register_GetDartBindPointerFunc");

  static void setDartBindPointer(Object object, Pointer<Void> pointer) {
    if (object is DartBaseClass) {
      object.cppObjectPointer = pointer;
    } else {
      UnitRCLog.error("obj ${object} not DartBaseClass");
    }
  }
  static final setDartBindPointerCFuntionPointer =
      Pointer.fromFunction<Void Function(Handle, Pointer<Void>)>(
          setDartBindPointer);
  static final registerSetDartBindPointerFunc = unitRCLib.lookupFunction<
          Void Function(
              Pointer<NativeFunction<Void Function(Handle, Pointer<Void>)>>),
          void Function(
              Pointer<NativeFunction<Void Function(Handle, Pointer<Void>)>>)>(
      "UnitRC_Register_SetDartBindPointerFunc");

  static Object? getDartNull() {
    return null;
  }
  static final getDartNullCFunctionPointer =
      Pointer.fromFunction<Handle Function()>(getDartNull);
  static final registerGetDartNullCFunctionPointer = unitRCLib.lookupFunction<
      Void Function(Pointer<NativeFunction<Handle Function()>>),
      void Function(
          Pointer<NativeFunction<Handle Function()>>)>("UnitRC_Register_GetDartNull");

  static bool isDartNull(Object? obj) {
    return obj == null;
  }
  static final isDartNullCFunctionPointer = Pointer.fromFunction<Bool Function(Handle)>(isDartNull, false);
  static final registerIsDartNullCFunctionPointer = unitRCLib.lookupFunction<
    Void Function(Pointer<NativeFunction<Bool Function(Handle)>>),
    void Function(Pointer<NativeFunction<Bool Function(Handle)>>)
  >("UnitRC_Register_IsDartNull");

  static Object getDartNullptr() {
    return nullptr;
  }
  static final getDartNullptrCFunctionPointer = Pointer.fromFunction<Handle Function()>(getDartNullptr);
  static final registerGetDartNullptrCFunctionPointer = unitRCLib.lookupFunction<
    Void Function(Pointer<NativeFunction<Handle Function()>>),
    void Function(Pointer<NativeFunction<Handle Function()>>)
  >("UnitRC_Register_GetDartNullptr");

  static int getDartHash(Object? obj) {
    return obj?.hashCode ?? 0;
  }
  static final getDartHashCFunctionPointer = Pointer.fromFunction<Int64 Function(Handle)>(getDartHash, 0);
  static final registerGetDartHashCFunctionPointer = unitRCLib.lookupFunction<
    Void Function(Pointer<NativeFunction<Int64 Function(Handle)>>),
    void Function(Pointer<NativeFunction<Int64 Function(Handle)>>)
  >("UnitRC_Register_GetDartHash");

  static int currentIsolate() {
    return Isolate.current.hashCode;
  }
  static final currentIsolateFP = Pointer.fromFunction<Int64 Function()>(currentIsolate, 0);
  static final registerCurrentIsolateFP = unitRCLib.lookupFunction<
    Void Function(Pointer<NativeFunction<Int64 Function()>>),
    void Function(Pointer<NativeFunction<Int64 Function()>>)
  >("UnitRC_Register_CurrentIsolate");

  static void notifyFieldUpdated(Object object, int index) {
    if (object is DartBaseClass) {
      object.notifyFieldUpdated(index);
    } else {
      UnitRCLog.error("obj ${object} not DartBaseClass");
    }
  }
  static final notifyFieldUpdatedFP = Pointer.fromFunction<Void Function(Handle, Int64)>(notifyFieldUpdated);
  static final registerNotifyFieldUpdatedFP = unitRCLib.lookupFunction<
    Void Function(Pointer<NativeFunction<Void Function(Handle, Int64)>>),
    void Function(Pointer<NativeFunction<Void Function(Handle, Int64)>>)
  >("UnitRC_Register_NotifyFieldUpdated");


  static void unregisterDartThread() {
    UnitRCLog.info("unregisterDartThread");
    isRegistered = false;
    unregisterDartThreadFP();
  }

  static final unregisterDartThreadFP = unitRCLib.lookupFunction<
      Void Function(),
      void Function()>("UnitRC_Unregister_DartThread");

  static bool isRegistered = false;
  static void registerNativeMethods() {
    if (isRegistered) {
      return;
    }
    isRegistered = true;
    UnitRCLog.info("registerNativeMethods");
    registerGetDartBindPointerFunc(getDartBindPointerFuncCFuntionPointer);
    registerSetDartBindPointerFunc(setDartBindPointerCFuntionPointer);
    registerGetDartNullCFunctionPointer(getDartNullCFunctionPointer);
    registerIsDartNullCFunctionPointer(isDartNullCFunctionPointer);
    registerGetDartNullptrCFunctionPointer(getDartNullptrCFunctionPointer);
    registerGetDartHashCFunctionPointer(getDartHashCFunctionPointer);
    registerCurrentIsolateFP(currentIsolateFP);
    registerNotifyFieldUpdatedFP(notifyFieldUpdatedFP);
    registerPortToCPP(createReceivePort());
    UnitRCLog.info("registerNativeMethods end");
  }
}
